#include <stdio.h>
#include <stdlib.h>
#include <stdbool.h>
#include <string.h>
#include <unistd.h>
#include <stddef.h>
#include <sys/socket.h>
#include <sys/syslog.h>
#include <netinet/in.h>
#include <netinet/tcp.h>
#include <fcntl.h>
#include <sys/ioctl.h>
#include <errno.h>
#include <signal.h>
#include <math.h>
#include <ctype.h>

#include <a2720.h>

#define MAXCONFIGLINELENGTH 256
#define MAXBUFFERLENGTH 9000

/* Write cvs tag into the objet file */
const char *TagName = "$Name:  $";

static volatile sig_atomic_t term_received;

static char serial_number[256], calibration_date[256], identification_name[256];

int load_configfile()
{
	FILE *file_ptr;
	char buffer[MAXCONFIGLINELENGTH];
	size_t i;

	file_ptr = fopen("/etc/a2720.cfg", "r");
	if (! file_ptr) {
		syslog(LOG_ERR, "%s", "ERROR fail to open file");
		return -1;
	}

	while ( !feof(file_ptr))
	{
		if (fgets(buffer,MAXCONFIGLINELENGTH, file_ptr) != NULL)
		{
			char *left = strtok(buffer,"=");
			char *right = strtok(NULL,"\n");

			char name[256], value[256];
			sscanf(left, "%s", name);

			for (i=0; i<strlen(right); ++i) {
				if (! isspace(right[i]))
						break;
			}
			strcpy(value, &right[i]);

			if (strcasecmp(name, "kp") == 0) {
			} else if (strcasecmp(name, "ki") == 0) {
			} else if (strcasecmp(name, "kff") == 0) {
			} else if (strcasecmp(name, "kd") == 0) {
			} else if (strcasecmp(name, "slewrate") == 0) {
			} else if (strcasecmp(name, "imin") == 0) {
			} else if (strcasecmp(name, "imax") == 0) {
			} else if (strcasecmp(name, "Iout-a0") == 0) {
			} else if (strcasecmp(name, "Iout-a1") == 0) {
			} else if (strcasecmp(name, "Iout-a2") == 0) {
			} else if (strcasecmp(name, "Iout-a3") == 0) {
			} else if (strcasecmp(name, "Iout-a4") == 0) {
			} else if (strcasecmp(name, "Iout-a5") == 0) {
			} else if (strcasecmp(name, "Vaux-a0") == 0) {
			} else if (strcasecmp(name, "Vaux-a1") == 0) {
			} else if (strcasecmp(name, "Vout-a0") == 0) {
			} else if (strcasecmp(name, "Vout-a1") == 0) {
			} else if (strcasecmp(name, "VDClink-a0") == 0) {
			} else if (strcasecmp(name, "VDClink-a1") == 0) {
			} else if (strcasecmp(name, "tempADC-a0") == 0) {
			} else if (strcasecmp(name, "tempADC-a1") == 0) {
			} else if (strcasecmp(name, "tempHeatsink-a0") == 0) {
			} else if (strcasecmp(name, "tempHeatsink-a1") == 0) {
			} else if (strcasecmp(name, "serial-number") == 0) {
				strcpy(serial_number, value);
			} else if (strcasecmp(name, "calibration-date") == 0) {
				strcpy(calibration_date, value);
			} else if (strcasecmp(name, "identification-name") == 0) {
				strcpy(identification_name, value);
			} else if (strcasecmp(name, "pulse-duration") == 0) {
			} else {
				fclose(file_ptr);
				syslog(LOG_ERR, "ERROR invalid configuration option: %s", value);
				return -1;
			}
		}
	}

	fclose(file_ptr);
	return 0;
}

static void sighup_handler(int sig)
{
	switch(sig) {
		case SIGHUP:
			syslog(LOG_INFO, "%s", "Reload configuration");
			if (load_configfile() != 0)
				abort();
			break;
		case SIGTERM:
			term_received = 1;
			break;
		default:
			syslog(LOG_ERR, "%s", "ERROR unhandled signal");
	}
}

ssize_t recv2(int fd, void *buffer, size_t n, int flags)
{
	ssize_t numRead;

retry:
	numRead = recv(fd, buffer, n, flags);

	if (numRead == -1) {
		if (errno == EINTR)
			goto retry;
		else
			return -1;
	}
	return numRead;
}

ssize_t sendn(int fd, const void *buffer, size_t n)
{
	ssize_t numWritten;
	size_t totWritten;
	const char *buf;

	buf = buffer;
	for (totWritten = 0; totWritten < n; ) {
		numWritten = send(fd, buf, n - totWritten, MSG_NOSIGNAL);

		if (numWritten <= 0) {
			if (numWritten == -1 && errno == EINTR
					&& !term_received)
				continue;
			else
				return -1;
		}
		totWritten += numWritten;
		buf += numWritten;
	}
	return totWritten;
}

int main(int argc , char *argv[])
{
	int exit_value = EXIT_SUCCESS;
	int serv_sockfd = -1, sess_sockfd = -1;
	ssize_t read_size = 0, write_size = 0;
	struct sockaddr_in serv_sockaddr;
	char rbuffer[MAXBUFFERLENGTH], command[MAXBUFFERLENGTH], wbuffer[MAXBUFFERLENGTH];
	int drv = -1;
	struct sigaction sa, sa_restarted;
	int reuseaddr=1;
	ssize_t ret;
	char *cptr, *vptr;
	struct a2720_config config;
	struct a2720_status status;
	int nagle_state = 1, cork_state;
	int found = -1;
	struct timeval timeout;
#ifndef NDEBUG
	int i;
#endif

	(void)argv;
	(void)argc;

	timeout.tv_sec = 60;
	timeout.tv_usec = 0;

	memset(&sa, 0, sizeof(struct sigaction));
	sa.sa_handler = sighup_handler;
	sigemptyset(&sa.sa_mask);
	if (sigaction(SIGTERM, &sa, 0) == -1)
		exit(EXIT_FAILURE);

	memset(&sa_restarted, 0, sizeof(struct sigaction));
	sa_restarted.sa_handler = sighup_handler;
	sa_restarted.sa_flags = SA_RESTART;
	sigemptyset(&sa_restarted.sa_mask);
	if (sigaction(SIGHUP, &sa_restarted, 0) == -1)
		exit(EXIT_FAILURE);

	if (load_configfile() != 0) {
		syslog(LOG_ERR, "%s", "ERROR Unable to load configuration file");
		exit(EXIT_FAILURE);
	}

	if (daemon(0,0) != 0) {
		fprintf(stderr, "Unable to fork process!");
		exit(EXIT_FAILURE);
	}

	openlog("a2720d", LOG_CONS | LOG_PID | LOG_NDELAY, LOG_LOCAL0);

	serv_sockfd = socket(AF_INET, SOCK_STREAM, 0);
	if (serv_sockfd < 0) {
		syslog(LOG_ERR, "%s", "ERROR creating socket");
		exit_value = EXIT_FAILURE;
		goto exit_closelog;
	}

	if (setsockopt(serv_sockfd, SOL_SOCKET, SO_REUSEADDR, &reuseaddr, sizeof(reuseaddr)) == -1) {
		syslog(LOG_ERR, "%s", "ERROR setting so_reuseaddr on socket");
		exit_value = EXIT_FAILURE;
		goto exit_closesocket;
	}

	memset(&serv_sockaddr,0,sizeof(struct sockaddr_in));
	serv_sockaddr.sin_family = AF_INET;
	serv_sockaddr.sin_addr.s_addr = htonl(INADDR_ANY);
	serv_sockaddr.sin_port = htons(10000);

	if (bind(serv_sockfd, (const struct sockaddr *)&serv_sockaddr, sizeof(struct sockaddr)) < 0) {
		syslog(LOG_ERR, "%s", "ERROR binding socket");
		exit_value = EXIT_FAILURE;
		goto exit_closesocket;
	}

	if (listen(serv_sockfd, 1) == -1) {
		syslog(LOG_ERR, "%s", "ERROR listening socket");
		exit_value = EXIT_FAILURE;
		goto exit_closesocket;
	}

	if ((drv = open("/dev/a2720", O_RDWR)) == -1) {
		syslog(LOG_ERR, "%s", "ERROR Unable to open /dev/a2720");
		exit_value = EXIT_FAILURE;
		goto exit_closesocket;
	}

	while (!term_received) {
		sess_sockfd = accept(serv_sockfd, 0, 0);
		if (sess_sockfd < 0) {
			if (! term_received)
				syslog(LOG_ERR, "%s", "ERROR accepting on socket");
			continue;
		}
		syslog(LOG_INFO, "%s", "Connection accepted");

		if (setsockopt(sess_sockfd, IPPROTO_TCP, TCP_NODELAY, &nagle_state, sizeof(nagle_state)) == -1) {
			syslog(LOG_ERR, "%s", "ERROR disabling nagle algorithm");
			close(sess_sockfd);
			exit_value = EXIT_FAILURE;
			goto exit_closedrv;
		}

		if (setsockopt(sess_sockfd, SOL_SOCKET, SO_RCVTIMEO, (char*)&timeout, sizeof(timeout)) < 0)  {
			syslog(LOG_ERR, "%s", "ERROR setting receive timeout");
			close(sess_sockfd);
			exit_value = EXIT_FAILURE;
			goto exit_closedrv;
		}

		if (setsockopt(sess_sockfd, SOL_SOCKET, SO_SNDTIMEO, (char*)&timeout, sizeof(timeout)) < 0) {
			syslog(LOG_ERR, "%s", "ERROR setting send timeout");
			close(sess_sockfd);
			exit_value = EXIT_FAILURE;
			goto exit_closedrv;
		}

		read_size = ret = 0;
		memset(&rbuffer,0,sizeof(rbuffer));
		while(!term_received && (ret = recv2(sess_sockfd, rbuffer+read_size, MAXBUFFERLENGTH-read_size, 0)) > 0) {
#ifndef NDEBUG
			syslog(LOG_ERR, "Just received %d total received %d", ret, read_size + ret);
			for (i=0; i<(read_size+ret); ++i)
				syslog(LOG_ERR, "0x%d %c", rbuffer[i], rbuffer[i]);
#endif
			read_size += ret;

			cork_state = 1;
			if (setsockopt(sess_sockfd, IPPROTO_TCP, TCP_CORK, &cork_state, sizeof(cork_state)) == -1) {
				syslog(LOG_ERR, "%s", "ERROR enabling corking");
				close(sess_sockfd);
				exit_value = EXIT_FAILURE;
				goto exit_closedrv;
			}

			while ((cptr = strchr(rbuffer, '\n'))) {
				/* Parse command */
				ptrdiff_t len = cptr - rbuffer + 1;
				memcpy(command, rbuffer, len);
				command[len] = '\0';
				read_size -= len;
				memmove(&rbuffer, &rbuffer[len], read_size);
				memset(&rbuffer[read_size],0, sizeof(rbuffer)-read_size);

				/* Execute command */
				if ((found = strncasecmp(command, "M", 1)) == 0
						&& ioctl(drv, A2720_GET_CONFIG, &config) == 0
						&& ioctl(drv, A2720_GET_STATUS, &status) == 0
						) {

					if ((found = strcasecmp(command, "MOFF\r\n")) == 0) {
						config.control = 0x0;
					} else if ((found = strcasecmp(command, "MON\r\n")) == 0) {
						config.control = 0xFFFF;
					} else if ((found = strcasecmp(command, "MRESET\r\n")) == 0) {
						config.reset = 0xFFFF;
					}

					if (found == 0) {
						if (ioctl(drv, A2720_SET_CONFIG, &config))
							sendn(sess_sockfd, "#NAK\r\n", 6);
						else
							sendn(sess_sockfd, "#AK\r\n", 5);
					} else {
						if (strncasecmp(command, "MR", 2) == 0) {
							if ((found = strcasecmp(command, "MRKP\r\n")) == 0) {
								write_size = snprintf(&wbuffer[1], MAXBUFFERLENGTH - 1, "%E", config.kp_coefficient);
							} else if ((found = strcasecmp(command, "MRKI\r\n")) == 0) {
								write_size = snprintf(&wbuffer[1], MAXBUFFERLENGTH - 1, "%E", config.ki_coefficient);
							} else if ((found = strcasecmp(command, "MRSR\r\n")) == 0) {
								/* PRU returns A/20us so we have to multiply by 50000 */
								write_size = snprintf(&wbuffer[1], MAXBUFFERLENGTH - 1, "%E", config.output_ramp_step * 50000);
							} else if ((found = strcasecmp(command, "MRKFF\r\n")) == 0) {
								write_size = snprintf(&wbuffer[1], MAXBUFFERLENGTH - 1, "%E", config.kff_coefficient);
							} else if ((found = strcasecmp(command, "MRKD\r\n")) == 0) {
								write_size = snprintf(&wbuffer[1], MAXBUFFERLENGTH - 1, "%E", config.kd_coefficient);
							} else if ((found = strcasecmp(command, "MRIMAX\r\n")) == 0) {
								write_size = snprintf(&wbuffer[1], MAXBUFFERLENGTH - 1, "%E", config.output_current_max);
							} else if ((found = strcasecmp(command, "MRIMIN\r\n")) == 0) {
								write_size = snprintf(&wbuffer[1], MAXBUFFERLENGTH - 1, "%E", config.output_current_min);
							} else if ((found = strcasecmp(command, "MRIA0\r\n")) == 0) {
								write_size = snprintf(&wbuffer[1], MAXBUFFERLENGTH - 1, "%E", config.a0_coefficient_scaling);
							} else if ((found = strcasecmp(command, "MRIA1\r\n")) == 0) {
								write_size = snprintf(&wbuffer[1], MAXBUFFERLENGTH - 1, "%E", config.a1_coefficient_scaling);
							} else if ((found = strcasecmp(command, "MRIA2\r\n")) == 0) {
								write_size = snprintf(&wbuffer[1], MAXBUFFERLENGTH - 1, "%E", config.a2_coefficient_scaling);
							} else if ((found = strcasecmp(command, "MRIA3\r\n")) == 0) {
								write_size = snprintf(&wbuffer[1], MAXBUFFERLENGTH - 1, "%E", config.a3_coefficient_scaling);
							} else if ((found = strcasecmp(command, "MRIA4\r\n")) == 0) {
								write_size = snprintf(&wbuffer[1], MAXBUFFERLENGTH - 1, "%E", config.a4_coefficient_scaling);
							} else if ((found = strcasecmp(command, "MRIA5\r\n")) == 0) {
								write_size = snprintf(&wbuffer[1], MAXBUFFERLENGTH - 1, "%E", config.a5_coefficient_scaling);
							} else if ((found = strcasecmp(command, "MRVXA0\r\n")) == 0) {
								write_size = snprintf(&wbuffer[1], MAXBUFFERLENGTH - 1, "%E", config.aux_a0);
							} else if ((found = strcasecmp(command, "MRVXA1\r\n")) == 0) {
								write_size = snprintf(&wbuffer[1], MAXBUFFERLENGTH - 1, "%E", config.aux_a1);
							} else if ((found = strcasecmp(command, "MRVA0\r\n")) == 0) {
								write_size = snprintf(&wbuffer[1], MAXBUFFERLENGTH - 1, "%E", config.output_voltage_a0);
							} else if ((found = strcasecmp(command, "MRVA1\r\n")) == 0) {
								write_size = snprintf(&wbuffer[1], MAXBUFFERLENGTH - 1, "%E", config.output_voltage_a1);
							} else if ((found = strcasecmp(command, "MRVDCA0\r\n")) == 0) {
								write_size = snprintf(&wbuffer[1], MAXBUFFERLENGTH - 1, "%E", config.dc_a0);
							} else if ((found = strcasecmp(command, "MRVDCA1\r\n")) == 0) {
								write_size = snprintf(&wbuffer[1], MAXBUFFERLENGTH - 1, "%E", config.dc_a1);
							} else if ((found = strcasecmp(command, "MRTA0\r\n")) == 0) {
								write_size = snprintf(&wbuffer[1], MAXBUFFERLENGTH - 1, "%E", config.heater_a0);
							} else if ((found = strcasecmp(command, "MRTA1\r\n")) == 0) {
								write_size = snprintf(&wbuffer[1], MAXBUFFERLENGTH - 1, "%E", config.heater_a1);
							} else if ((found = strcasecmp(command, "MRTHA0\r\n")) == 0) {
								write_size = snprintf(&wbuffer[1], MAXBUFFERLENGTH - 1, "%E", config.heatsink_a0);
							} else if ((found = strcasecmp(command, "MRTHA1\r\n")) == 0) {
								write_size = snprintf(&wbuffer[1], MAXBUFFERLENGTH - 1, "%E", config.heatsink_a1);
							} else if ((found = strcasecmp(command, "MRIS\r\n")) == 0) {
								write_size = snprintf(&wbuffer[1], MAXBUFFERLENGTH - 1, "%f", config.output_current_setpoint);
							} else if ((found = strcasecmp(command, "MRI\r\n")) == 0) {
								write_size = snprintf(&wbuffer[1], MAXBUFFERLENGTH - 1, "%.6f", TRUNCF(status.output_current));
							} else if ((found = strcasecmp(command, "MRVX\r\n")) == 0) {
								write_size = snprintf(&wbuffer[1], MAXBUFFERLENGTH - 1, "%.6f", TRUNCF(config.aux_a0 + (config.aux_a1 * (float)status.auxiliary_voltage)));
							} else if ((found = strcasecmp(command, "MRVDC\r\n")) == 0) {
								write_size = snprintf(&wbuffer[1], MAXBUFFERLENGTH - 1, "%.6f", TRUNCF(config.dc_a0 + (config.dc_a1 * (float)status.dclink_voltage)));
							} else if ((found = strcasecmp(command, "MRV\r\n")) == 0) {
								write_size = snprintf(&wbuffer[1], MAXBUFFERLENGTH - 1, "%.6f", TRUNCF(config.output_voltage_a0 + (config.output_voltage_a1 * (float)status.output_voltage)));
							} else if ((found = strcasecmp(command, "MRTH\r\n")) == 0) {
								write_size = snprintf(&wbuffer[1], MAXBUFFERLENGTH - 1, "%.6f", TRUNCF(config.heatsink_a0 + (config.heatsink_a1 * (float)status.heatsink_temperature)));
							} else if ((found = strcasecmp(command, "MRT\r\n")) == 0) {
								write_size = snprintf(&wbuffer[1], MAXBUFFERLENGTH - 1, "%.6f", TRUNCF(config.heater_a0 + (config.heater_a1 * (float)status.heater_temperature)));
							} else if ((found = strcasecmp(command, "MRST\r\n")) == 0) {
								write_size = snprintf(&wbuffer[1], MAXBUFFERLENGTH - 1, "%d", status.iostatus);
							} else if ((found = strcasecmp(command, "MRSN\r\n")) == 0) {
								write_size = snprintf(&wbuffer[1], MAXBUFFERLENGTH - 1, "%s", serial_number);
							} else if ((found = strcasecmp(command, "MRID\r\n")) == 0) {
								write_size = snprintf(&wbuffer[1], MAXBUFFERLENGTH - 1, "%s", identification_name);
							} else if ((found = strcasecmp(command, "MRCD\r\n")) == 0) {
								write_size = snprintf(&wbuffer[1], MAXBUFFERLENGTH - 1, "%s", calibration_date);
							} else if ((found = strcasecmp(command, "MRPD\r\n")) == 0) {
								write_size = snprintf(&wbuffer[1], MAXBUFFERLENGTH - 1, "%E", config.pulse_duration * 0.02);
							}
							
							if (found == 0) {
								wbuffer[0] = '#';
								wbuffer[write_size + 1] = '\r';
								wbuffer[write_size + 2] = '\n';
								sendn(sess_sockfd, wbuffer, write_size + 3);
							}
						} else if (strncasecmp(command, "MW", 2) == 0
								&& (vptr = strchr(command, ':')) && strlen(vptr+1) > 1) {
							char *nptr = vptr + 1, *endptr;
							if ((found = strncasecmp(command, "MWKP:", 5)) == 0) {
								config.kp_coefficient = strtof(nptr, &endptr);
							} else if ((found = strncasecmp(command, "MWKI:", 5)) == 0) {
								config.ki_coefficient = strtof(nptr, &endptr);
							} else if ((found = strncasecmp(command, "MWSR:", 5)) == 0) {
								config.output_ramp_step = strtof(nptr, &endptr) / 50000;
							} else if ((found = strncasecmp(command, "MWKFF:", 6)) == 0) {
								config.kff_coefficient = strtof(nptr, &endptr);
							} else if ((found = strncasecmp(command, "MWKD:", 5)) == 0) {
								config.kd_coefficient = strtof(nptr, &endptr);
							} else if ((found = strncasecmp(command, "MWIMAX:", 7)) == 0) {
								config.output_current_max = strtof(nptr, &endptr);
							} else if ((found = strncasecmp(command, "MWIMIN:", 7)) == 0) {
								config.output_current_min = strtof(nptr, &endptr);
							} else if ((found = strncasecmp(command, "MWIA0:", 6)) == 0) {
								config.a0_coefficient_scaling = strtof(nptr, &endptr);
							} else if ((found = strncasecmp(command, "MWIA1:", 6)) == 0) {
								config.a1_coefficient_scaling = strtof(nptr, &endptr);
							} else if ((found = strncasecmp(command, "MWIA2:", 6)) == 0) {
								config.a2_coefficient_scaling = strtof(nptr, &endptr);
							} else if ((found = strncasecmp(command, "MWIA3:", 6)) == 0) {
								config.a3_coefficient_scaling = strtof(nptr, &endptr);
							} else if ((found = strncasecmp(command, "MWIA4:", 6)) == 0) {
								config.a4_coefficient_scaling = strtof(nptr, &endptr);
							} else if ((found = strncasecmp(command, "MWIA5:", 6)) == 0) {
								config.a5_coefficient_scaling = strtof(nptr, &endptr);
							} else if ((found = strncasecmp(command, "MWVXA0:", 7)) == 0) {
								config.aux_a0 = strtof(nptr, &endptr);
							} else if ((found = strncasecmp(command, "MWVXA1:", 7)) == 0) {
								config.aux_a1 = strtof(nptr, &endptr);
							} else if ((found = strncasecmp(command, "MWVA0:", 6)) == 0) {
								config.output_voltage_a0 = strtof(nptr, &endptr);
							} else if ((found = strncasecmp(command, "MWVA1:", 6)) == 0) {
								config.output_voltage_a1 = strtof(nptr, &endptr);
							} else if ((found = strncasecmp(command, "MWVDCA0:", 8)) == 0) {
								config.dc_a0 = strtof(nptr, &endptr);
							} else if ((found = strncasecmp(command, "MWVDCA1:", 8)) == 0) {
								config.dc_a1 = strtof(nptr, &endptr);
							} else if ((found = strncasecmp(command, "MWTA0:", 6)) == 0) {
								config.heater_a0 = strtof(nptr, &endptr);
							} else if ((found = strncasecmp(command, "MWTA1:", 6)) == 0) {
								config.heater_a1 = strtof(nptr, &endptr);
							} else if ((found = strncasecmp(command, "MWTHA0:", 7)) == 0) {
								config.heatsink_a0 = strtof(nptr, &endptr);
							} else if ((found = strncasecmp(command, "MWTHA1:", 7)) == 0) {
								config.heatsink_a1 = strtof(nptr, &endptr);
							} else if ((found = strncasecmp(command, "MWI:", 4)) == 0) {
								config.output_current_setpoint = strtof(nptr, &endptr);
							} else if ((found = strncasecmp(command, "MWPD:", 5)) == 0) {
								config.pulse_duration = strtof(nptr, &endptr) / 0.02;
							}

							if (found == 0) {
							       	if (errno == ERANGE || strcmp(nptr, endptr) == 0) {
									sendn(sess_sockfd, "#NAK\r\n", 6);
								} else {
									if (ioctl(drv, A2720_SET_CONFIG, &config))
										sendn(sess_sockfd, "#NAK\r\n", 6);
									else
										sendn(sess_sockfd, "#AK\r\n", 5);
								}
							}
						}
					}
				}
				
				if (found != 0) {
					/* Unknow command */
					sendn(sess_sockfd, "#NAK\r\n", 6);
				}

				memset(&command,0,sizeof(command));
			}

			cork_state = 0;
			if (setsockopt(sess_sockfd, IPPROTO_TCP, TCP_CORK, &cork_state, sizeof(cork_state)) == -1) {
				syslog(LOG_ERR, "%s", "ERROR disabling corking");
				close(sess_sockfd);
				exit_value = EXIT_FAILURE;
				goto exit_closedrv;
			}

			if (read_size == MAXBUFFERLENGTH) {
				sendn(sess_sockfd, "#NAK\r\n", 6);
				read_size = 0;
				memset(&rbuffer,0,sizeof(rbuffer));
			}
		}

		if (ret == 0)
			syslog(LOG_INFO, "%s", "Connection closed");
		else /* ret== -1 */ {
			if (term_received) {
				syslog(LOG_INFO, "%s", "Exiting");
			} else {
				syslog(LOG_ERR, "Connection error (%d %s)", errno,  strerror(errno));
			}
		}
		close(sess_sockfd);
	}

exit_closedrv:
	close(drv);
	drv = -1;

exit_closesocket:
	close(serv_sockfd);
	serv_sockfd = -1;

exit_closelog:
	closelog();
	return exit_value; 
}

